# Add tests here

import unittest
from elara import LRU


class RunTests(unittest.TestCase):
    def setUp(self):
        self.lru = LRU()
        self.lru.push("key1")
        self.lru.push("key2")

    def test_push(self):
        self.assertEqual(self.lru.all(), ["key2", "key1"])

    def test_peek(self):
        self.assertEqual(self.lru.peek(), "key2")

    def test_touch(self):
        self.lru.touch("key1")
        self.assertEqual(self.lru.peek(), "key1")

    def test_clear(self):
        self.lru.clear()
        self.assertEqual(self.lru.all(), [])

    def test_rem(self):
        self.lru.rem("key2")
        self.assertEqual(self.lru.all(), ["key1"])

    def test_pop(self):
        self.assertEqual(self.lru.pop(), "key1")
        self.assertEqual(len(self.lru.all()), 1)



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""

# Add String specific functions


def setnx(self, key, data):
    if not self.exists(key):
        self.set(key, data)
        return True
    else:
        return False


def append(self, key, data):
    if self.exists(key):
        if isinstance(data, str):
            value = self.db[key]
            self.db[key] = value + data
            self._autocommit()
            return self.db[key]
        else:
            return False
    else:
        return False


def getset(self, key, data):
    if self.exists(key):
        old_data = self.db[key]
    else:
        return False
    self.set(key, data)
    return old_data


def mget(self, keys):
    result = []
    for key in keys:
        if self.exists(key):
            result.append(self.get(key))
    return result


def mset(self, dict):
    for key in dict:
        if isinstance(key, str):
            self.set(key, dict[key])
    return True


def msetnx(self, dict):
    for key in dict:
        self.setnx(key, dict[key])
    return True


def slen(self, key):
    if self.exists(key):
        return len(self.db[key])
    return -1



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""


class LRU:
    def __init__(self) -> None:
        self.cache = []
        self.size = 0

    # Push a new key into the cache
    def push(self, key):
        if key in self.cache:
            self.touch(key)
        else:
            self.cache.insert(0, key)
            self.size += 1

    # Pop the least recently used key (end of the cache list)
    def pop(self):
        if len(self.cache) == 0:
            return False
        else:
            val = self.cache[-1]
            del self.cache[-1]
            self.size -= 1
            return val

    def rem(self, key):
        if key in self.cache:
            self.cache.remove(key)
            self.size -= 1
        else:
            return False

    # Peek the most recently used key (start of the cache list)
    def peek(self):
        return self.cache[0]

    def all(self):
        return self.cache

    # Bring key to the front of the cache
    def touch(self, key):
        if key in self.cache:
            if self.cache[0] == key:
                pass
            else:
                self.cache.remove(key)
                self.push(key)
        else:
            return False

    def clear(self):
        self.cache = []
        self.size = 0

    def print(self):
        print(self.cache)



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""


# Add all custom exception classes here


class FileAccessError(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"Error -> {self.message}"


class FileKeyError(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"Error -> {self.message}"


class LoadChecksumError(Exception):
    pass


class LoadIncompatibleDB(Exception):
    pass



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""

# shared operations
from cryptography.fernet import Fernet
from .elarautil import Util
import json
import os
from .exceptions import FileAccessError, FileKeyError


def retdb(self):
    if self.key:
        return Util.read_and_decrypt(self)
    else:
        return Util.read_plain_db(self)


def retmem(self):
    return self.db


def retkey(self):
    return self.key


def commit(self):
    self._dump()


def exportdb(self, export_path, sort=True):
    db = self.retdb()
    new_export_path = os.path.expanduser(export_path)
    try:
        json.dump(db, open(new_export_path, "wt"), indent=4, sort_keys=sort)
    except Exception:
        raise FileAccessError("Store JSON error. File path might be wrong")


def exportmem(self, export_path, sort=True):
    db = self.retmem()
    new_export_path = os.path.expanduser(export_path)
    try:
        json.dump(db, open(new_export_path, "wt"), indent=4, sort_keys=sort)
    except Exception:
        raise FileAccessError("Store JSON error. File path might be wrong")


def exportkeys(self, export_path, keys=[], sort=True):
    db = {}
    for key in keys:
        if isinstance(key, str) and self.exists(key):
            db[key] = self.get(key)

    new_export_path = os.path.expanduser(export_path)
    try:
        json.dump(db, open(new_export_path, "wt"), indent=4, sort_keys=sort)
    except Exception:
        raise FileAccessError("Store JSON error. File path might be wrong")


def securedb(self, key_path=None):
    if key_path is None:
        raise FileAccessError("Please specify a valid key path")
    elif self.key:
        self.updatekey(self, key_path)
    else:
        new_key_path = os.path.expanduser(key_path)
        self.db = self.retdb()  # Store the current contents

        if os.path.exists(new_key_path):
            if os.stat(new_key_path).st_size == 0:
                Util.keygen(new_key_path)
            else:
                # os.remove(new_key_path)
                f = open(new_key_path, "r+")
                f.truncate(0)
                f.close()
                Util.keygen(new_key_path)
        else:
            Util.keygen(new_key_path)

        self.key = Util.readkey(new_key_path)
        Util.encrypt_and_store(self)
        return True


# Incomplete function
def updatekey(self, key_path=None):
    if key_path is None:
        raise FileAccessError("Please specify a valid key path")
    elif self.key:
        new_key_path = os.path.expanduser(key_path)
        self.db = self.retdb()
        if os.path.exists(new_key_path):
            if os.stat(new_key_path).st_size == 0:
                Util.keygen(new_key_path)
            else:
                # os.remove(new_key_path)
                f = open(new_key_path, "r+")
                f.truncate(0)
                f.close()
                Util.keygen(new_key_path)

        else:
            Util.keygen(new_key_path)

        # clear db file and encrypt contents with new key
        f = open(self.path, "r+")
        f.truncate(0)
        f.close
        self.key = Util.readkey(new_key_path)
        Util.encrypt_and_store(self)

    else:
        raise FileKeyError("Update key Failed")



# Add tests here

import unittest
import elara


class RunTests(unittest.TestCase):
    def setUp(self):
        self.db = elara.exe("test.db", False)

    def test_setnx(self):
        self.db.set("key1", "value1")
        self.assertEqual(self.db.setnx("key1", "value"), False)
        self.assertEqual(self.db.setnx("key", "value"), True)
        self.assertEqual(self.db.setnx("key", "value"), False)

    def test_append(self):
        self.db.set("key", "value")
        self.assertEqual(self.db.append("key", "data"), "valuedata")
        self.assertEqual(self.db.append("key", 1), False)

    def test_getset(self):
        self.db.set("key", "value")
        self.assertEqual(self.db.getset("key", "newvalue"), "value")
        self.assertEqual(self.db.getset("key", 1), "newvalue")
        self.assertEqual(self.db.get("key"), 1)

    def test_mget(self):
        self.db.set("key", "value")
        self.db.set("key1", 3)
        keys = ["key", "key1", "key2"]
        self.assertEqual(self.db.mget(keys), ["value", 3])
        self.db.set("key2", "value2")
        self.assertEqual(self.db.mget(keys), ["value", 3, "value2"])
        self.assertNotEqual(self.db.mget(keys), ["value", "value2", 3])

    def test_mset(self):
        new_dict = {"key1": [1, 2, 3], 1: "value", "key2": "value2"}
        self.db.mset(new_dict)
        self.assertEqual(self.db.mget(["key1", 1, "key2"]), [[1, 2, 3], "value2"])

    def test_msetnx(self):
        self.db.set("key", "value")
        new_dict = {"key": [1, 2, 3], "key2": "value2", "key3": "value3"}
        self.db.msetnx(new_dict)
        self.assertEqual(
            self.db.mget(["key", "key3", "key2"]), ["value", "value3", "value2"]
        )



# Add tests here

import unittest
import elara
import os


class RunTests(unittest.TestCase):
    def setUp(self):
        self.db = elara.exe("test.db", False)

    def test_exe(self):
        res = elara.exe("test.db", False)
        assert res is not None

    def test_store_restore_data(self):
        db = elara.exe("test_store.db")
        db.set(
            "test_key",
            'test_data:"ði ıntəˈnæʃənəl fəˈnɛtık əsoʊsiˈeıʃn Y [ˈʏpsilɔn], Yen [jɛn], Yoga [ˈjoːgɑ]"',
        )
        db.commit()
        db_load = elara.exe("test_store.db")
        recov_data = db_load.get("test_key")
        self.assertEqual(
            recov_data,
            'test_data:"ði ıntəˈnæʃənəl fəˈnɛtık əsoʊsiˈeıʃn Y [ˈʏpsilɔn], Yen [jɛn], Yoga [ˈjoːgɑ]"',
        )
        # cleanup database files
        os.remove("test_store.db")

    def test_store_restore_data_secure(self):
        db = elara.exe_secure("test_enc.db")
        db.set(
            "test_key",
            'test_data:"ði ıntəˈnæʃənəl fəˈnɛtık əsoʊsiˈeıʃn Y [ˈʏpsilɔn], Yen [jɛn], Yoga [ˈjoːgɑ]"',
        )
        db.commit()
        db_load = elara.exe_secure("test_enc.db")
        recov_data = db_load.get("test_key")
        self.assertEqual(
            recov_data,
            'test_data:"ði ıntəˈnæʃənəl fəˈnɛtık əsoʊsiˈeıʃn Y [ˈʏpsilɔn], Yen [jɛn], Yoga [ˈjoːgɑ]"',
        )

        # cleanup database files
        os.remove("test_enc.db")
        os.remove("edb.key")

    def test_get(self):
        self.db.clear()
        self.db.db["key"] = "test"
        res = self.db.db["key"]
        self.assertEqual(res, "test")

    def test_set(self):
        self.db.clear()
        self.db.set("key", "test")
        self.assertEqual(self.db.get("key"), "test")

    def test_rem(self):
        self.db.clear()
        self.db.set("key", "test")
        assert "key" in self.db.db
        self.db.rem("key")
        assert "key" not in self.db.db

    def test_clear(self):
        self.db.set("key", "test")
        self.assertEqual(self.db.clear(), True)
        self.assertEqual(self.db.db, {})

    def test_cull(self):
        self.db.clear()
        self.db.set("one", 1)
        self.db.set("two", 2)
        self.db.set("three", 3)
        self.db.set("four", 4)
        self.db.cull(25)
        self.assertEqual(len(self.db.retmem()), 3)
        self.db.cull(100)
        self.assertEqual(len(self.db.retmem()), 0)
        self.assertEqual(self.db.cull(-1), False)
        self.assertEqual(self.db.cull(101), False)

    def test_incr(self):
        self.db.clear()
        self.db.set("one", 1)
        self.db.incr("one")
        self.assertEqual(self.db.get("one"), 2)
        self.db.incr("one", 3.6)
        self.assertEqual(self.db.get("one"), 5.6)
        self.db.incr("one", 0.0003)
        self.assertEqual(self.db.get("one"), 5.600)
        self.db.incr("one", -1)
        self.assertEqual(self.db.get("one"), 4.600)

    def test_decr(self):
        self.db.clear()
        self.db.set("one", 1.35)
        self.db.decr("one")
        self.assertEqual(self.db.get("one"), 0.35)
        self.db.decr("one")
        self.assertEqual(self.db.get("one"), -0.65)



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""
import os
from .elarautil import Util
from .lru import LRU


class Elara:

    from .strings import setnx, append, getset, mget, mset, msetnx, slen
    from .lists import (
        lnew,
        lpush,
        lextend,
        lindex,
        lrange,
        lrem,
        lpop,
        llen,
        lappend,
        lexists,
        linsert,
    )
    from .hashtables import hnew, hadd, haddt, hget, hpop, hkeys, hvals, hexists, hmerge
    from .shared import (
        retmem,
        retdb,
        retkey,
        commit,
        exportdb,
        exportkeys,
        exportmem,
        securedb,
        updatekey,
    )

    def __init__(self, path, commitdb, key_path=None):
        self.path = os.path.expanduser(path)
        self.commitdb = commitdb
        self.lru = LRU()
        # this is in place to prevent opening incompatible databases between versions of the storage format
        self.db_format_version = 0x0001

        # Since key file is generated first, invalid token error for pre existing open dbs

        # Load the database key
        if not key_path == None:
            new_key_path = os.path.expanduser(key_path)
            if os.path.exists(new_key_path):
                file = open(new_key_path, "rb")
                self.key = file.read()
                file.close()
            else:
                self.key = None
        else:
            self.key = None

        # Load the data
        if os.path.exists(path):
            self._load()
        else:
            self.db = {}

    def _load(self):
        if self.key:
            self.db = Util.read_and_decrypt(self)
        else:
            self.db = Util.read_plain_db(self)

    def _dump(self):
        if self.key:
            Util.encrypt_and_store(self)  # Enclose in try-catch
        else:
            Util.store_plain_db(self)

    def _autocommit(self):
        if self.commitdb:
            self._dump()

    def set(self, key, value):
        if isinstance(key, str):
            self.db[key] = value
            self.lru.push(key)
            self._autocommit()
            return True
        else:
            raise Exception

    def get(self, key):
        try:
            self.lru.touch(key)
            return self.db[key]
        except KeyError:
            return None

    def rem(self, key):
        self.lru.rem(key)
        del self.db[key]
        self._autocommit()
        return True

    def clear(self):
        self.lru.clear()
        self.db = {}
        self._autocommit()
        return True

    def exists(self, key):
        self.lru.touch(key)
        return key in self.db

    def cull(self, percentage=10):
        if 0 <= percentage <= 100:
            count = int((percentage / 100) * (self.numkeys()))
            print("final count", count)
            for i in range(0, count):
                key = self.lru.pop()
                del self.db[key]
            self._autocommit()
            return True
        else:
            return False

    def getkeys(self):
        return self.lru.all()

    def numkeys(self):
        return self.lru.size

    def incr(self, key, val=1):
        if self.exists(key):
            data = self.get(key)
            if isinstance(data, (int, float)):
                data += val
                data = round(data, 3)
                self.set(key, data)
            else:
                return False
        else:
            return False

    def decr(self, key, val=1):
        if self.exists(key):
            data = self.get(key)
            if isinstance(data, (int, float)):
                data -= val
                data = round(data, 3)
                self.set(key, data)
            else:
                return False
        else:
            return False



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""

# Add list specific functions


def lnew(self, key):
    if isinstance(key, str):
        self.db[key] = []
        self.lru.push(key)
        self._autocommit()
        return True
    else:
        raise Exception


def lpush(self, key, value):
    if self.exists(key):
        self.db[key].append(value)
        self._autocommit()
        return True
    else:
        return False


def lextend(self, key, list_data):
    if self.exists(key):
        self.db[key].extend(list_data)
        self._autocommit()
        return True
    else:
        return False


def lindex(self, key, index):
    if self.exists(key) and len(self.db[key]) > index:
        return self.db[key][index]
    else:
        return False


def lrange(self, key, start=None, end=None):
    self.lru.touch(key)
    return self.db[key][start:end]


def lrem(self, key, value):
    if self.exists(key) and len(self.db[key]) > 0:
        try:
            self.db[key].remove(value)
        except ValueError:
            return False
        self._autocommit()
        return True
    else:
        return False


def lpop(self, key, pos=-1):
    len = self.llen(key)
    if self.exists(key) and pos < len:
        if pos < -1:
            return False
        else:
            value = self.db[key][pos]
            del self.db[key][pos]
            self._autocommit()
            return value
    else:
        return False


def llen(self, key):
    if self.exists(key):
        return len(self.db[key])
    else:
        return -1


def lappend(self, key, pos, more):
    try:
        tmp = self.db[key][pos]
        self.db[key][pos] = tmp + more
        self.lru.touch(key)
    except:
        return False
    self._autocommit()
    return True


def lexists(self, key, value):
    # modify and return index
    if self.exists(key):
        return value in self.db[key]
    else:
        return False


def linsert(self, key, value, index):
    pass



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""

# Hash table operations


def hnew(self, key):
    if isinstance(key, str):
        self.db[key] = {}
        self.lru.push(key)
        self._autocommit()
        return True
    else:
        raise Exception


def hadd(self, key, dict_key, value):
    if self.exists(key):
        self.db[key][dict_key] = value
        self._autocommit()
        return True
    else:
        return False


def haddt(self, key, tuple):
    if self.exists(key):
        self.db[key][tuple[0]] = tuple[1]
        self._autocommit()
        return True
    else:
        return False


def hget(self, key, dict_key):
    if self.exists(key):
        return self.db[key][dict_key]
    else:
        return False


def hpop(self, key, dict_key):
    if self.exists(key):
        value = self.db[key][dict_key]
        del self.db[key][dict_key]
        self._autocommit()
        return value
    else:
        return False


def hkeys(self, key):
    self.lru.touch(key)
    return self.db[key].keys()


def hvals(self, key):
    self.lru.touch(key)
    return self.db[key].values()


def hexists(self, key, dict_key):
    self.lru.touch(key)
    return dict_key in self.db[key]


def hmerge(self, key, new_dict):
    self.lru.touch(key)
    first = self.db[key]
    first.update(new_dict)
    self._autocommit()
    return True



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""
from typing import Dict

import msgpack
import os
from zlib import crc32

from cryptography.fernet import Fernet

from .exceptions import (
    FileAccessError,
    FileKeyError,
    LoadChecksumError,
    LoadIncompatibleDB,
)


class Util:
    @staticmethod
    def check_mag(mag):
        return mag == b"ELDB"

    @staticmethod
    def check_encrypted(version):
        # if msb of version number is set the db is encrypted
        return (version & (1 << 15)) != 0

    @staticmethod
    def read_plain_db(obj) -> Dict:
        with open(obj.path, "rb") as fctx:
            if not Util.check_mag(fctx.read(4)):
                raise FileAccessError("File magic number not known")
            version = int.from_bytes(fctx.read(2), "little", signed=False)
            # check for encryption before trying anything
            if Util.check_encrypted(version):
                raise FileAccessError("This file is encrypted, run in secure mode")
            checksum = int.from_bytes(fctx.read(4), "little", signed=False)
            data = fctx.read()
            calculated_checksum = crc32(data)
            if calculated_checksum != checksum:
                raise LoadChecksumError(
                    f"calculated checksum: {calculated_checksum} is different from stored checksum {checksum}"
                )
            elif version != obj.db_format_version:
                raise LoadIncompatibleDB(
                    f"db format version {version} is incompatible with {obj.db_format_version}"
                )
            try:
                curr_db = msgpack.unpackb(data)
            except FileNotFoundError:
                raise FileAccessError("File not found")
            return curr_db

    @staticmethod
    def store_plain_db(obj):
        with open(obj.path, "wb") as fctx:
            try:
                data = msgpack.packb(obj.db)
                buffer = b"ELDB"
                buffer += obj.db_format_version.to_bytes(2, "little")
                buffer += (crc32(data)).to_bytes(4, "little")
                buffer += data
                fctx.write(buffer)
            except FileExistsError:
                raise FileAccessError("File already exists")

    @staticmethod
    def read_and_decrypt(obj):
        if obj.key:
            fernet = Fernet(obj.key)
            try:
                with open(obj.path, "rb") as fctx:
                    if not Util.check_mag(fctx.read(4)):
                        raise FileAccessError("File magic number not known")
                    version = int.from_bytes(fctx.read(2), "little")
                    if not Util.check_encrypted(version):
                        raise FileAccessError(
                            "File is marked not encrypted, you might have a corrupt db"
                        )
                    checksum = int.from_bytes(fctx.read(4), "little")
                    encrypted_data = fctx.read()
                    calculated_checksum = crc32(encrypted_data)
                    if calculated_checksum != checksum:
                        raise LoadChecksumError(
                            f"calculated checksum: {calculated_checksum} is different from stored checksum {checksum}"
                        )
            except FileNotFoundError:
                raise FileAccessError("File open & read error")
            decrypted_data = fernet.decrypt(encrypted_data)
            return msgpack.unpackb(decrypted_data)
        else:
            return None

    @staticmethod
    def encrypt_and_store(obj):
        if obj.key:
            fernet = Fernet(obj.key)
            db_snapshot = msgpack.packb(obj.db)
            buffer = b"ELDB"
            # set version msb
            buffer += (obj.db_format_version | 1 << 15).to_bytes(2, "little")
            encrypted_data = fernet.encrypt(db_snapshot)
            buffer += crc32(encrypted_data).to_bytes(4, "little")
            buffer += encrypted_data
            try:
                with open(obj.path, "wb") as file:
                    file.write(buffer)
                    return True
            except FileExistsError:
                raise FileAccessError("File exists")
        else:
            return False

    @staticmethod
    def keygen(path):
        try:
            key = Fernet.generate_key()
        except Exception:
            raise FileKeyError("Key generation error")
        try:
            with open(path, "wb") as file:
                file.write(key)
            file.close()
            return True
        except Exception:
            raise FileAccessError("File open & write error")

    @staticmethod
    def loadkey(path):
        key_path = os.path.expanduser(path)
        # print(key_path)

        if os.path.exists(key_path):
            if os.stat(key_path).st_size == 0:
                Util.keygen(key_path)
                # else: key exists in file; use that
        else:
            # create file and store keygen
            Util.keygen(key_path)

    @staticmethod
    def readkey(path):
        key_path = os.path.expanduser(path)
        if os.path.exists(key_path):
            file = open(key_path, "rb")
            key = file.read()
            file.close()
            return key
        else:
            key = None
            return key



# Add tests here

import unittest
import elara


class RunTests(unittest.TestCase):
    def setUp(self):
        self.db = elara.exe("test.db", False)
        self.db.lnew("newlist")

    def test_lpush(self):
        self.assertEqual(len(self.db.get("newlist")), 0)
        self.db.lpush("newlist", 3)
        self.assertEqual(len(self.db.get("newlist")), 1)
        self.db.lpush("newlist", 4)
        self.assertEqual(len(self.db.get("newlist")), 2)
        new_list = self.db.get("newlist")
        self.assertEqual(new_list[1], 4)
        self.db.lpush("newlist", "string")
        new_list = self.db.get("newlist")
        self.assertEqual(new_list[2], "string")

    def test_lextend(self):
        self.db.lpush("newlist", 3)
        self.assertEqual(len(self.db.get("newlist")), 1)
        new_list = [4, 5]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(len(self.db.get("newlist")), 3)
        self.assertEqual(self.db.lextend("newlist2", new_list), False)

    def test_lindex(self):
        self.db.lpush("newlist", 3)
        new_list = [4, 5]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(self.db.lindex("newlist", 0), 3)
        self.assertEqual(self.db.lindex("newlist", 5), False)

    def test_lrange(self):
        new_list = [4, 5, 6, 7, 8, 9]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(self.db.lrange("newlist", 2, 4), [6, 7])
        self.assertEqual(self.db.lrange("newlist", 10, 20), [])

    def test_lrem(self):
        new_list = ["cat", "dog", "bird"]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(self.db.lrem("newlist", "dog"), True)
        self.assertEqual(self.db.lrem("newlist", 20), False)

    def test_lpop(self):
        new_list = [4, 5, 6, 7, 8, 9]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(self.db.lpop("newlist", -2), False)
        self.assertEqual(self.db.lpop("newlist"), 9)
        self.assertEqual(len(self.db.get("newlist")), len(new_list) - 1)
        self.assertEqual(self.db.lpop("list"), False)

    def test_llen(self):
        new_list = [4, 5, 6, 7, 8, 9]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(self.db.llen("newlist"), len(new_list))
        self.assertEqual(self.db.llen("list"), -1)

    def test_lappend(self):
        new_list = [4, 5, 6, 7, 8, 9]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(self.db.lappend("newlist", 0, 3), True)
        self.assertEqual(self.db.lindex("newlist", 0), 7)
        self.assertEqual(self.db.lappend("list", 0, 3), False)

    def test_lexists(self):
        new_list = [4, 5, 6, 7, 8, 9]
        self.assertEqual(self.db.lextend("newlist", new_list), True)
        self.assertEqual(self.db.lexists("newlist", 5), True)
        self.assertEqual(self.db.lexists("newlist", 20), False)
        self.assertEqual(self.db.lexists("list", 0), False)



from distutils.core import setup

with open("README.rst") as f:
    long_description = f.read()

setup(
    name="elara",
    packages=["elara"],
    version="0.4.0",
    license="three-clause BSD",
    description="Elara DB is an easy to use, lightweight NoSQL database written for python that can also be used as a fast in-memory cache for JSON-serializable data. Includes various methods and features to manipulate data structures in-memory, protect database files and export data.",
    long_description=long_description,
    author="Saurabh Pujari",
    author_email="saurabhpuj99@gmail.com",
    url="https://github.com/saurabh0719/elara",
    keywords=[
        "database",
        "key-value",
        "storage",
        "file storage",
        "key-value database",
        "nosql",
        "nosql database",
        "cache",
        "file cache",
    ],
    project_urls={
        "Documentation": "https://github.com/saurabh0719/elara#readme",
        "Source": "https://github.com/saurabh0719/elara",
    },
    install_requires=["cryptography", "msgpack"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Database",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
    ],
)



"""
Copyright (c) 2021, Saurabh Pujari
All rights reserved.

This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
"""
from .elara import Elara
from .elarautil import Util
from .lru import LRU


def exe(path, commitdb=False):
    return Elara(path, commitdb)


def exe_secure(path, commitdb=False, key_path="edb.key"):
    Util.loadkey(key_path)
    return Elara(path, commitdb, key_path)
